@@ -46,7 +46,7 @@ import java.util.List;
  * A parser plugin which adapts the JSR Antlr Parser to the Groovy runtime
  *
  * @author <a href="mailto:jstrachan@protique.com">James Strachan</a>
- * @version $Revision: 11961 $
+ * @version $Revision: 13714 $
  */
 public class AntlrParserPlugin extends ASTHelper implements ParserPlugin, GroovyTokenTypes {
 
@@ -474,12 +474,13 @@ public class AntlrParserPlugin extends ASTHelper implements ParserPlugin, Groovy
     		name = identifier(node);
     	}
     	ClassNode exception = ClassHelper.make(name);
+    	configureAST(exception, node);
     	list.add(exception);
     	AST next = node.getNextSibling();
     	if (next!=null) throwsList(next, list);
     }
-    
-    protected void methodDef(AST methodDef) {
+
+	protected void methodDef(AST methodDef) {
         List annotations = new ArrayList();
         AST node = methodDef.getFirstChild();
         
@@ -680,13 +681,33 @@ public class AntlrParserPlugin extends ASTHelper implements ParserPlugin, Groovy
             // let's pass along any other modifiers we need
             fieldModifiers |= (modifiers & flags);
             fieldNode.setModifiers(fieldModifiers);
+            fieldNode.setSynthetic(true);
+            
+            // in the case that there is already a field, we would
+            // like to use that field, instead of the default field
+            // for the property
+            FieldNode storedNode = classNode.getDeclaredField(fieldNode.getName());
+            if (storedNode!=null && !classNode.hasProperty(name)) {
+            	fieldNode = storedNode;
+            	// we remove it here, because addProperty will add it
+            	// again and we want to avoid it showing up multiple 
+            	// times in the fields list.
+        		classNode.getFields().remove(storedNode);
+            }
             
             PropertyNode propertyNode = new PropertyNode(fieldNode, modifiers, null, null);
             configureAST(propertyNode, fieldDef);
             classNode.addProperty(propertyNode);
-        }
-        else {
-            fieldNode.setModifiers(modifiers);
+        } else {
+        	fieldNode.setModifiers(modifiers);
+        	// if there is a property of that name, then a field of that
+        	// name already exists, which means this new field here should
+        	// be used instead of the field the property originally has.
+        	PropertyNode pn = classNode.getProperty(name);
+        	if (pn!=null && pn.getField().isSynthetic()) {
+        		classNode.getFields().remove(pn.getField());
+        		pn.setField(fieldNode);
+        	}
             classNode.addField(fieldNode);
         }
     }
@@ -1028,6 +1049,7 @@ public class AntlrParserPlugin extends ASTHelper implements ParserPlugin, Groovy
     
             collectionExpression = expression(collectionNode);
             forParameter = new Parameter(type,variable);
+            configureAST(forParameter,variableNode);
         }
 
         final AST node = inNode.getNextSibling();
@@ -1085,7 +1107,7 @@ public class AntlrParserPlugin extends ASTHelper implements ParserPlugin, Groovy
             node = node.getNextSibling();
         }
         if (isType(TYPE, node)) {
-            type = makeType(node);
+            type = makeTypeWithArguments(node);
             node = node.getNextSibling();
         }
 
@@ -1332,7 +1354,7 @@ public class AntlrParserPlugin extends ASTHelper implements ParserPlugin, Groovy
                 return methodCallExpression(node);
 
             case LITERAL_new:
-                return constructorCallExpression(node.getFirstChild());
+                return constructorCallExpression(node);
 
             case CTOR_CALL:
                 return specialConstructorCallExpression(node,ClassNode.THIS);
@@ -1708,6 +1730,7 @@ public class AntlrParserPlugin extends ASTHelper implements ParserPlugin, Groovy
         } else {
             methodName = new ConstantExpression(identifier(mNode));
         }
+        configureAST(methodName,mNode);
         MethodPointerExpression methodPointerExpression = new MethodPointerExpression(objectExpression, methodName);
         configureAST(methodPointerExpression, node);
         return methodPointerExpression;
@@ -1916,7 +1939,7 @@ public class AntlrParserPlugin extends ASTHelper implements ParserPlugin, Groovy
     }
 
     protected Expression dotExpression(AST node) {
-        // let's decide if this is a propery invocation or a method call
+        // let's decide if this is a property invocation or a method call
         AST leftNode = node.getFirstChild();
         if (leftNode != null) {
             AST identifierNode = leftNode.getNextSibling();
@@ -2036,7 +2059,7 @@ public class AntlrParserPlugin extends ASTHelper implements ParserPlugin, Groovy
     
     protected Expression constructorCallExpression(AST node) {
         AST constructorCallNode = node;
-        ClassNode type = buildName(constructorCallNode);
+        ClassNode type = makeTypeWithArguments(constructorCallNode);
 
         if (isType(CTOR_CALL, node) || isType(LITERAL_new, node)) {
             node = node.getFirstChild();
@@ -2345,8 +2368,9 @@ public class AntlrParserPlugin extends ASTHelper implements ParserPlugin, Groovy
                 " or type: " + getTokenName(TYPE_LOWER_BOUNDS));       
     }
     
-    private GenericsType makeGenericsArgumentType(AST rootNode) {
+    private GenericsType makeGenericsArgumentType(AST typeArgument) {
         GenericsType gt;
+        AST rootNode = typeArgument.getFirstChild();
         if (isType(WILDCARD_TYPE,rootNode)) {
             ClassNode base = ClassHelper.makeWithoutCaching("?");
             if (rootNode.getNextSibling()!=null) {
@@ -2366,7 +2390,7 @@ public class AntlrParserPlugin extends ASTHelper implements ParserPlugin, Groovy
             ClassNode argument = makeTypeWithArguments(rootNode);
             gt = new GenericsType(argument);
         }
-        configureAST(gt, rootNode);
+        configureAST(gt, typeArgument);
         return gt;
     }
     
@@ -2384,7 +2408,7 @@ public class AntlrParserPlugin extends ASTHelper implements ParserPlugin, Groovy
         
         while (typeArgument != null) {
             assertNodeType(TYPE_ARGUMENT, typeArgument);            
-            GenericsType gt = makeGenericsArgumentType(typeArgument.getFirstChild());
+            GenericsType gt = makeGenericsArgumentType(typeArgument);
             typeArgumentList.add(gt);
             typeArgument = typeArgument.getNextSibling();
         }
@@ -2423,7 +2447,7 @@ public class AntlrParserPlugin extends ASTHelper implements ParserPlugin, Groovy
             ClassNode type = makeType(typeParameter);
             
             GenericsType gt = new GenericsType(type, makeGenericsBounds(typeNode,TYPE_UPPER_BOUNDS),null);
-            configureAST(gt, typeNode);
+            configureAST(gt, typeParameter);
             
             ret.add(gt);
             typeParameter = typeParameter.getNextSibling();
@@ -2473,7 +2497,9 @@ public class AntlrParserPlugin extends ASTHelper implements ParserPlugin, Groovy
         }
         else if (isType(INDEX_OP, node) || isType(ARRAY_DECLARATOR, node)) {
             AST child = node.getFirstChild();
-            return buildName(child).makeArray();
+            answer = buildName(child).makeArray();
+            configureAST(answer, node);
+            return answer;
         }
         else {
             String identifier = node.getText();
@@ -2481,9 +2507,12 @@ public class AntlrParserPlugin extends ASTHelper implements ParserPlugin, Groovy
         }
         AST nextSibling = node.getNextSibling();
         if (isType(INDEX_OP, nextSibling) || isType(ARRAY_DECLARATOR, node)) {
-            return answer.makeArray();
+        	answer = answer.makeArray();
+        	configureAST(answer, node);
+            return answer;
         }
         else {
+        	configureAST(answer, node);
             return answer;
         }
     }
