@@ -53,7 +53,7 @@ import java.util.*;
  * 
  * @author <a href="mailto:james@coredevelopers.net">James Strachan</a>
  * @author Jochen Theodorou
- * @version $Revision: 12038 $
+ * @version $Revision: 13567 $
  */
 public class ClassNode extends AnnotatedNode implements Opcodes {
 
@@ -102,7 +102,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
     private ClassNode superClass;
     protected boolean isPrimaryNode;
     
-    // use this to synchronize access for the lazy intit
+    // use this to synchronize access for the lazy init
     protected Object lazyInitLock = new Object();
 
     // clazz!=null when resolved
@@ -308,7 +308,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
     /**
      * Returns a list containing MethodNode objects for
      * each method in the class represented by this ClassNode
-     */    
+     */
     public List getMethods() {
         if (!lazyInitDone) lazyClassInit();
         if (redirect!=null) return redirect().getMethods();
@@ -464,6 +464,18 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         return node;
     }
 
+    public boolean hasProperty(String name) {
+    	return getProperty(name)!=null;
+    }
+    
+    public PropertyNode getProperty(String name) {
+    	for (Iterator iter = getProperties().iterator(); iter.hasNext();) {
+            PropertyNode pn = (PropertyNode) iter.next();
+            if (pn.getName().equals(name)) return pn;
+        }
+        return null;   	
+    }
+
     public void addConstructor(ConstructorNode node) {
         node.setDeclaringClass(this);
         redirect().constructors.add(node);
@@ -482,10 +494,10 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
     }
 
     /**
-     * IF a method with the given name and parameters is already defined then it is returned
+     * If a method with the given name and parameters is already defined then it is returned
      * otherwise the given method is added to this node. This method is useful for
      * default method adding like getProperty() or invokeMethod() where there may already
-     * be a method defined in a class and  so the default implementations should not be added
+     * be a method defined in a class and so the default implementations should not be added
      * if already present.
      */
     public MethodNode addMethod(String name,
@@ -495,7 +507,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
                                 ClassNode[] exceptions,
                                 Statement code) {
         MethodNode other = getDeclaredMethod(name, parameters);
-        // lets not add duplicate methods
+        // let's not add duplicate methods
         if (other != null) {
             return other;
         }
@@ -504,6 +516,22 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         return node;
     }
 
+    /**
+     * @see #getDeclaredMethod(String, Parameter[])
+     */
+    public boolean hasDeclaredMethod(String name, Parameter[] parameters) {
+        MethodNode other = getDeclaredMethod(name, parameters);
+        return other != null;
+    }
+
+    /**
+     * @see #getMethod(String, Parameter[])
+     */
+    public boolean hasMethod(String name, Parameter[] parameters) {
+        MethodNode other = getMethod(name, parameters);
+        return other != null;
+    }
+
     /**
      * Adds a synthetic method as part of the compilation process
      */
@@ -564,10 +592,32 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         }
     }
 
-    public FieldNode getField(String name) {
+    /**
+     * Finds a field matching the given name in this class.
+     *
+     * @param name the name of the field of interest
+     * @return the method matching the given name and parameters or null
+     */
+    public FieldNode getDeclaredField(String name) {
         return (FieldNode) redirect().fieldIndex.get(name);
     }
 
+    /**
+     * Finds a field matching the given name in this class or a parent class.
+     *
+     * @param name the name of the field of interest
+     * @return the method matching the given name and parameters or null
+     */
+    public FieldNode getField(String name) {
+        ClassNode node = this;
+        while (node != null) {
+            FieldNode fn = node.getDeclaredField(name);
+            if (fn != null) return fn;
+            node = node.getSuperClass();
+        }
+        return null;
+    }
+
     /**
      * @return the field node on the outer class or null if this is not an
      *         inner class
@@ -655,6 +705,8 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
     }
 
     /**
+     * Finds a method matching the given name and parameters in this class.
+     *
      * @return the method matching the given name and parameters or null
      */
     public MethodNode getDeclaredMethod(String name, Parameter[] parameters) {
@@ -668,6 +720,23 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         return null;
     }
 
+    /**
+     * Finds a method matching the given name and parameters in this class
+     * or any parent class.
+     *
+     * @return the method matching the given name and parameters or null
+     */
+    public MethodNode getMethod(String name, Parameter[] parameters) {
+        List list = getMethods(name);
+        for (Iterator iter = list.iterator(); iter.hasNext();) {
+            MethodNode method = (MethodNode) iter.next();
+            if (parametersEqual(method.getParameters(), parameters)) {
+                return method;
+            }
+        }
+        return null;
+    }
+
     /**
      * @return true if this node is derived from the given class node
      */
@@ -688,17 +757,17 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
      *         i.e. it implements GroovyObject
      */
     public boolean isDerivedFromGroovyObject() {
-        return implementsInterface(GroovyObject.class.getName());
+        return implementsInterface(ClassHelper.make(GroovyObject.class));
     }
 
     /**
-     * @param name the fully qualified name of the interface
+     * @param classNode the class node for the interface
      * @return true if this class or any base class implements the given interface
      */
-    public boolean implementsInterface(String name) {
+    public boolean implementsInterface(ClassNode classNode) {
         ClassNode node = redirect();
         do {
-            if (node.declaresInterface(name)) {
+            if (node.declaresInterface(classNode)) {
                 return true;
             }
             node = node.getSuperClass();
@@ -708,14 +777,30 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
     }
 
     /**
-     * @param name the fully qualified name of the interface
+     * @param classNode the class node for the interface
      * @return true if this class declares that it implements the given interface
+     * or if one of its interfaces extends directly or indirectly the interface
      */
-    public boolean declaresInterface(String name) {
+    public boolean declaresInterface(ClassNode classNode) {
         ClassNode[] interfaces = redirect().getInterfaces();
+        if (declaresInterfaceDirect(interfaces, classNode)) return true;
+        List superInterfaces = Arrays.asList(interfaces);
+        while (superInterfaces.size() > 0) {
+            List keep = new ArrayList();
+            for (int i = 0; i < superInterfaces.size(); i++) {
+                ClassNode cn = (ClassNode) superInterfaces.get(i);
+                if (cn.declaresInterface(classNode)) return true;
+                keep.addAll(Arrays.asList(cn.getInterfaces()));
+            }
+            superInterfaces = keep;
+        }
+        return false;
+    }
+
+    private boolean declaresInterfaceDirect(ClassNode[] interfaces, ClassNode classNode) {
         int size = interfaces.length;
         for (int i = 0; i < size; i++) {
-            if (interfaces[i].getName().equals(name)) {
+            if (interfaces[i].equals(classNode)) {
                 return true;
             }
         }
@@ -910,7 +995,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
         }
         ClassNode node = this;
         do {
-            for (Iterator iter = getDeclaredMethods(name).iterator(); iter.hasNext();) {
+            for (Iterator iter = getMethods(name).iterator(); iter.hasNext();) {
                 MethodNode method = (MethodNode) iter.next();
                 if (method.getParameters().length == count) {
                     return true;
@@ -933,7 +1018,7 @@ public class ClassNode extends AnnotatedNode implements Opcodes {
             // TODO this won't strictly be true when using list expansion in argument calls
             count = tuple.getExpressions().size();
         }
-        for (Iterator iter = getDeclaredMethods(name).iterator(); iter.hasNext();) {
+        for (Iterator iter = getMethods(name).iterator(); iter.hasNext();) {
             MethodNode method = (MethodNode) iter.next();
             if (method.getParameters().length == count && method.isStatic()) {
                 return true;
